@@ -5,29 +5,55 @@
 
 #include "local.h"
 
-INT triang_split(tr,ce,le)
-lfit *tr;
+void solve(A,b,d) /* this is crude! A organized by column. */
+double *A, *b;
+int d;
+{ int i, j, k;
+  double piv;
+  for (i=0; i<d; i++)
+  { piv = A[(d+1)*i];
+    for (j=i; j<d; j++) A[j*d+i] /= piv;
+    b[i] /= piv;
+    for (j=0; j<d; j++) if (j != i)
+    { piv = A[i*d+j];
+      A[i*d+j] = 0;
+      for (k=i+1; k<d; k++)
+        A[k*d+j] -= piv*A[k*d+i];
+      b[j] -= piv*b[i];
+    }
+  }
+}
+
+void triang_guessnv(nvm,ncm,vc,d,mk)
+int *nvm, *ncm, *vc, d, mk;
+{ *nvm = *ncm = mk*d;
+  *vc = d+1;
+  return;         
+}
+
+int triang_split(lf,ce,le)
+lfit *lf;
 double *le;
-INT *ce;
-{ INT d, i, j, k, nts, vc;
+Sint *ce;
+{ int d, i, j, k, nts, vc;
   double di, dfx[MXDIM];
-  nts = 0; d = tr->mi[MDIM]; vc = d+1;
+  nts = 0; d = lf->fp.d; vc = d+1;
   for (i=0; i<d; i++)
     for (j=i+1; j<=d; j++)
     { for (k=0; k<d; k++)
-        dfx[k] = evptx(tr,ce[i],k)-evptx(tr,ce[j],k);
-      di = rho(dfx,tr->sca,d,KSPH,NULL);
-      le[i*vc+j] = le[j*vc+i] = di/MIN(tr->h[ce[i]],tr->h[ce[j]]);
-      nts = nts || le[i*vc+j]>tr->dp[DCUT];
+        dfx[k] = evptx(&lf->fp,ce[i],k)-evptx(&lf->fp,ce[j],k);
+      di = rho(dfx,lf->lfd.sca,d,KSPH,NULL);
+      le[i*vc+j] = le[j*vc+i] = di/MIN(lf->fp.h[ce[i]],lf->fp.h[ce[j]]);
+      nts = nts || le[i*vc+j]>cut(&lf->evs);
     }
   return(nts);
 }
 
 void resort(pv,xev,dig)
 double *xev;
-INT *pv, *dig;
+int *pv, *dig;
 { double d0, d1, d2;
-  INT i;
+  int i;
   d0 = d1 = d2 = 0;
   for (i=0; i<3; i++)
   { d0 += (xev[3*pv[11]+i]-xev[3*pv[1]+i])*(xev[3*pv[11]+i]-xev[3*pv[1]+i]);
@@ -51,15 +77,16 @@ INT *pv, *dig;
   }
 }
 
-void triang_grow(des,tr,ce,ct,term)
+void triang_grow(des,lf,ce,ct,term)
 design *des;
-lfit *tr;
-INT *ce, *ct, *term;
+lfit *lf;
+Sint *ce, *ct, *term;
 { double le[(1+MXDIM)*(1+MXDIM)], ml;
-  INT pv[(1+MXDIM)*(1+MXDIM)], nce[1+MXDIM], d, i, j, im, jm, vc, dig[6];
+  int d, i, j, im, jm, vc, pv[(1+MXDIM)*(1+MXDIM)], dig[6];
+  Sint nce[1+MXDIM];
   if (lf_error) return;
-  d = tr->mi[MDIM]; vc = d+1;
-  if (!triang_split(tr,ce,le))
+  d = lf->fp.d; vc = d+1;
+  if (!triang_split(lf,ce,le))
   { if (ct != NULL)
     { for (i=0; i<vc; i++) term[*ct*vc+i] = ce[i];
       (*ct)++;
@@ -71,54 +98,53 @@ INT *ce, *ct, *term;
     for (i=0; i<d; i++)
       for (j=i+1; j<vc; j++)
         if (le[i*vc+j]>ml) { ml = le[i*vc+j]; im = i; jm = j; }
-    pv[0] = newsplit(des,tr,ce[im],ce[jm],0);
+    pv[0] = newsplit(des,lf,(int)ce[im],(int)ce[jm],0);
     for (i=0; i<vc; i++) nce[i] = ce[i];
-    nce[im] = pv[0]; triang_grow(des,tr,nce,ct,term); nce[im] = ce[im];
-    nce[jm] = pv[0]; triang_grow(des,tr,nce,ct,term);
+    nce[im] = pv[0]; triang_grow(des,lf,nce,ct,term); nce[im] = ce[im];
+    nce[jm] = pv[0]; triang_grow(des,lf,nce,ct,term);
     return;
   }
 
   for (i=0; i<d; i++)
     for (j=i+1; j<=d; j++)
       pv[i*vc+j] = pv[j*vc+i]
-        = newsplit(des,tr,ce[i],ce[j],le[i*vc+j]<=tr->dp[DCUT]);
+        = newsplit(des,lf,(int)ce[i],(int)ce[j],le[i*vc+j]<=cut(&lf->evs));
   for (i=0; i<=d; i++) /* corners */
   { for (j=0; j<=d; j++) nce[j] = (j==i) ? ce[i] : pv[i*vc+j];
-    triang_grow(des,tr,nce,ct,term);
+    triang_grow(des,lf,nce,ct,term);
   }
   
   if (d==2) /* center for d=2 */
   { nce[0] = pv[5]; nce[1] = pv[2]; nce[2] = pv[1];
-    triang_grow(des,tr,nce,ct,term);
+    triang_grow(des,lf,nce,ct,term);
   }
   if (d==3) /* center for d=3 */
-  { resort(pv,vdptr(tr->xxev),dig);
+  { resort(pv,evp(&lf->fp),dig);
     nce[0] = dig[0]; nce[1] = dig[1];
-    nce[2] = dig[2]; nce[3] = dig[4]; triang_grow(des,tr,nce,ct,term);
-    nce[2] = dig[5]; nce[3] = dig[3]; triang_grow(des,tr,nce,ct,term);
-    nce[2] = dig[2]; nce[3] = dig[5]; triang_grow(des,tr,nce,ct,term);
-    nce[2] = dig[4]; nce[3] = dig[3]; triang_grow(des,tr,nce,ct,term);
+    nce[2] = dig[2]; nce[3] = dig[4]; triang_grow(des,lf,nce,ct,term);
+    nce[2] = dig[5]; nce[3] = dig[3]; triang_grow(des,lf,nce,ct,term);
+    nce[2] = dig[2]; nce[3] = dig[5]; triang_grow(des,lf,nce,ct,term);
+    nce[2] = dig[4]; nce[3] = dig[3]; triang_grow(des,lf,nce,ct,term);
   }
-  if (d==1) return;
 }
 
 void triang_descend(tr,xa,ce)
 lfit *tr;
 double *xa;
-INT *ce;
+Sint *ce;
 { double le[(1+MXDIM)*(1+MXDIM)], ml;
-  INT d, vc, i, j, pv[(1+MXDIM)*(1+MXDIM)], im, jm;
+  int d, vc, i, j, im, jm, pv[(1+MXDIM)*(1+MXDIM)];
   design *des;
   des = NULL;
   if (!triang_split(tr,ce,le)) return;
-  d = tr->mi[MDIM]; vc = d+1;
+  d = tr->fp.d; vc = d+1;
 
   if (d>3) /* split longest edge */
   { ml = 0;
     for (i=0; i<d; i++)
       for (j=i+1; j<vc; j++)
         if (le[i*vc+j]>ml) { ml = le[i*vc+j]; im = i; jm = j; }
-    pv[0] = newsplit(des,tr,ce[im],ce[jm],0);
+    pv[0] = newsplit(des,tr,(int)ce[im],(int)ce[jm],0);
     if (xa[im]>xa[jm])
     { xa[im] -= xa[jm]; xa[jm] *= 2; ce[jm] = pv[0]; }
     else
@@ -130,7 +156,7 @@ INT *ce;
   for (i=0; i<d; i++)
     for (j=i+1; j<=d; j++)
       pv[i*vc+j] = pv[j*vc+i]
-        = newsplit(des,tr,ce[i],ce[j],le[i*d+j]<=tr->dp[DCUT]);
+        = newsplit(des,tr,(int)ce[i],(int)ce[j],le[i*d+j]<=cut(&tr->evs));
   for (i=0; i<=d; i++) if (xa[i]>=0.5) /* in corner */
   { for (j=0; j<=d; j++)
     { if (i!=j) ce[j] = pv[i*vc+j];
@@ -140,7 +166,7 @@ INT *ce;
     triang_descend(tr,xa,ce);
     return;
   }
-  if (d==1) { lfERROR(("weights sum to < 1")); }
+  if (d==1) { ERROR(("weights sum to < 1")); }
   if (d==2) /* center */
   { ce[0] = pv[5]; xa[0] = 1-2*xa[0];
     ce[1] = pv[2]; xa[1] = 1-2*xa[1];
@@ -148,8 +174,8 @@ INT *ce;
     triang_descend(tr,xa,ce);
   }
   if (d==3) /* center */
-  { double z; INT dig[6];
-    resort(pv,vdptr(tr->xxev),dig);
+  { double z; int dig[6];
+    resort(pv,evp(&tr->fp),dig);
     ce[0] = dig[0]; ce[1] = dig[1];
     xa[0] *= 2; xa[1] *= 2; xa[2] *= 2; xa[3] *= 2;
     if (xa[0]+xa[2]>=1)
@@ -176,26 +202,27 @@ INT *ce;
     triang_descend(tr,xa,ce);
 } }
 
-void covrofdata(lf,V,mn) /* covar of data; mean in mn */
-lfit *lf;
+void covrofdata(lfd,V,mn) /* covar of data; mean in mn */
+lfdata *lfd;
 double *V, *mn;
-{ INT d, i, j, k;
+{ int d, i, j, k;
   double s;
-  s = 0; d = lf->mi[MDIM];
+  s = 0; d = lfd->d;
   for (i=0; i<d*d; i++) V[i] = 0;
-  for (i=0; i<lf->mi[MN]; i++)
-  { s += prwt(lf,i);
+  for (i=0; i<lfd->n; i++)
+  { s += prwt(lfd,i);
     for (j=0; j<d; j++)
       for (k=0; k<d; k++)
-        V[j*d+k] += prwt(lf,i)*(datum(lf,j,i)-mn[j])*(datum(lf,k,i)-mn[k]);
+        V[j*d+k] += prwt(lfd,i)*(datum(lfd,j,i)-mn[j])*(datum(lfd,k,i)-mn[k]);
   }
   for (i=0; i<d*d; i++) V[i] /= s;
 }
 
-INT intri(x,w,xev,xa,d) /* is x in triangle bounded by xd[0..d-1]? */
+int intri(x,w,xev,xa,d) /* is x in triangle bounded by xd[0..d-1]? */
 double *x, *xev, *xa;
-INT d, *w;
-{ INT i, j;
+Sint *w;
+int d;
+{ int i, j;
   double eps, *r, xd[MXDIM*MXDIM];
   eps = 1.0e-10;
   r = &xev[w[d]*d];
@@ -210,40 +237,43 @@ INT d, *w;
   return(1);
 }
 
-void triang_start(des,tr) /* Triangulation with polyhedral start */
+void triang_start(des,lf) /* Triangulation with polyhedral start */
 design *des;
-lfit *tr;
-{ INT i, j, k, n, d, nc, nvm, ncm, vc, *ce, ed[1+MXDIM];
+lfit *lf;
+{
+  int i, j, k, n, d, nc, nvm, ncm, vc;
+  Sint *ce, ed[1+MXDIM];
   double V[MXDIM*MXDIM], P[MXDIM*MXDIM], sigma, z[MXDIM], xa[1+MXDIM], *xev;
-  xev = vdptr(tr->xxev);
-  d = tr->mi[MDIM]; n = tr->mi[MN]; tr->nv = nc = 0;
+  xev = evp(&lf->fp);
+  d = lf->lfd.d; n = lf->lfd.n;
+  lf->fp.nv = nc = 0;
 
-  guessnv(&nvm,&ncm,&vc,tr->dp,tr->mi);
-  trchck(tr,nvm,ncm,d,des->p,vc);
+  triang_guessnv(&nvm,&ncm,&vc,d,mk(&lf->evs));
+  trchck(lf,nvm,ncm,vc);
 
-  ce = tr->ce;
-  for (j=0; j<d; j++) xev[j] = tr->pc.xbar[j];
-  tr->nv = 1;
-  covrofdata(tr,V,tr->pc.xbar); /* fix this with scaling */
+  ce = lf->evs.ce;
+  for (j=0; j<d; j++) xev[j] = lf->pc.xbar[j];
+  lf->fp.nv = 1;
+  covrofdata(&lf->lfd,V,xev); /* fix this with scaling */
   eig_dec(V,P,d);
 
   for (i=0; i<d; i++) /* add vertices +- 2sigma*eigenvect */
   { sigma = sqrt(V[i*(d+1)]);
     for (j=0; j<d; j++)
-      xev[tr->nv*d+j] = xev[j]-2*sigma*P[j*d+i];
-    tr->nv++;
+      xev[lf->fp.nv*d+j] = xev[j]-2*sigma*P[j*d+i];
+    lf->fp.nv++;
     for (j=0; j<d; j++)
-      xev[tr->nv*d+j] = xev[j]+2*sigma*P[j*d+i];
-    tr->nv++;
+      xev[lf->fp.nv*d+j] = xev[j]+2*sigma*P[j*d+i];
+    lf->fp.nv++;
   }
 
   for (i=0; i<n; i++) /* is point i inside? */
   { ed[0] = 0;
     for (j=0; j<d; j++)
     { z[j] = 0;
-      for (k=0; k<d; k++) z[j] += P[k*d+j]*(datum(tr,k,i)-xev[k]);
+      for (k=0; k<d; k++) z[j] += P[k*d+j]*(datum(&lf->lfd,k,i)-xev[k]);
       ed[j+1] = 2*j+1+(z[j]>0);
-      for (k=0; k<d; k++) z[j] = datum(tr,j,i);
+      for (k=0; k<d; k++) z[j] = datum(&lf->lfd,j,i);
     }
     k = intri(z,ed,xev,xa,d);
     if (xa[0]<0)
@@ -262,23 +292,22 @@ lfit *tr;
     }
   }
 
-  for (i=0; i<tr->nv; i++)
-  { des->vfun(des,tr,i);
+  for (i=0; i<lf->fp.nv; i++)
+  { des->vfun(des,lf,i);
     if (lf_error) return;
-    tr->s[i] = 0;
+    lf->evs.s[i] = 0;
   }
   for (i=0; i<nc; i++)
-  { for (j=0; j<=d; j++) ed[j] = tr->ce[i*vc+j];
-    triang_grow(des,tr,&tr->ce[i*vc],(INT *)NULL,(INT *)NULL);
-  }
-  tr->nce = nc;
+    triang_grow(des,lf,&ce[i*vc],NULL,NULL);
+  lf->evs.nce = nc;
 }
 
 double triang_cubicint(v,vv,w,d,nc,xxa)
 double *v, *vv, *xxa;
-INT d, *w, nc;
+int d, nc;
+Sint *w;
 { double sa, lb, *vert0, *vert1, *vals0, *vals1, deriv0, deriv1;
-  INT i, j, k;
+  int i, j, k;
   if (nc==1) /* linear interpolate */
   { sa = 0;
     for (i=0; i<=d; i++) sa += xxa[i]*vv[i];
@@ -309,10 +338,11 @@ INT d, *w, nc;
 
 double triang_clotoch(xev,vv,ce,p,xxa)
 double *xev, *vv, *xxa;
-INT *ce, p;
+int p;
+Sint *ce;
 { double cfo[3], cfe[3], cg[9], *va, *vb, *vc,
     l0, nm[3], na, nb, nc, *xl, *xr, *xz, d0, d1, lb, dlt, gam;
-  INT i, w[3], cfl, cfr;
+  int i, w[3], cfl, cfr;
   if (p==1)
     return(xxa[0]*vv[0]+xxa[1]*vv[1]+xxa[2]*vv[2]);
   if (xxa[2]<=MIN(xxa[0],xxa[1]))
@@ -391,47 +421,55 @@ INT *ce, p;
   return(cubic_interp(gam,cfo[0],cfe[0],d0,d1));
 }
 
-INT triang_getvertexvals(lf,vv,i,what)
-lfit *lf;
+int triang_getvertexvals(fp,evs,vv,i,what)
+fitpt *fp;
+evstruc *evs;
 double *vv;
-INT i, what;
+int i, what;
 { double dx, P, le, vl[1+MXDIM], vh[1+MXDIM];
-  INT d, il, ih, j, nc;
-  d = lf->mi[MDIM];
-  if (lf->s[i]==0) return(exvval(lf,vv,i,d,what,0));
+  int d, il, ih, j, nc;
+  d = fp->d;
+  if (evs->s[i]==0) return(exvval(fp,vv,i,d,what,0));
 
-  il = lf->lo[i]; nc = triang_getvertexvals(lf,vl,il,what);
-  ih = lf->hi[i]; nc = triang_getvertexvals(lf,vh,ih,what);
+  il = evs->lo[i]; nc = triang_getvertexvals(fp,evs,vl,il,what);
+  ih = evs->hi[i]; nc = triang_getvertexvals(fp,evs,vh,ih,what);
   vv[0] = (vl[0]+vh[0])/2;
   if (nc==1) return(nc);
   P = 1.5*(vh[0]-vl[0]);
   le = 0.0;
   for (j=0; j<d; j++)
-  { dx = evptx(lf,ih,j)-evptx(lf,il,j);
+  { dx = evptx(fp,ih,j)-evptx(fp,il,j);
     vv[0] += dx*(vl[j+1]-vh[j+1])/8;
     vv[j+1] = (vl[j+1]+vh[j+1])/2;
     P -= 1.5*dx*vv[j+1];
     le += dx*dx;
   }
   for (j=0; j<d; j++)
-    vv[j+1] += P*(evptx(lf,ih,j)-evptx(lf,il,j))/le;
+    vv[j+1] += P*(evptx(fp,ih,j)-evptx(fp,il,j))/le;
   return(nc);
 }
 
-double triang_int(tr,x,what)
-lfit *tr;
+double triang_int(lf,x,what)
+lfit *lf;
 double *x;
-INT what;
-{ INT d, vc, i, j, k, *ce, nc, nce[1+MXDIM];
+int what;
+{
+  int d, i, j, k, vc, nc;
+  Sint *ce, nce[1+MXDIM];
   double xa[1+MXDIM], vv[(1+MXDIM)*(1+MXDIM)], lb;
-  d = tr->mi[MDIM]; vc = d+1;
-  ce = tr->ce;
+fitpt *fp;
+evstruc *evs;
+fp = &lf->fp;
+evs= &lf->evs;
+
+  d = fp->d; vc = d+1;
+  ce = evs->ce;
   i = 0;
-  while ((i<tr->nce) && (!intri(x,&ce[i*vc],vdptr(tr->xxev),xa,d))) i++;
-  if (i==tr->nce) return(NOSLN);
+  while ((i<evs->nce) && (!intri(x,&ce[i*vc],evp(fp),xa,d))) i++;
+  if (i==evs->nce) return(NOSLN);
   i *= vc;
   for (j=0; j<vc; j++) nce[j] = ce[i+j];
-  triang_descend(tr,xa,nce);
+  triang_descend(lf,xa,nce);
 
   /* order the vertices -- needed for asymmetric interptr */
   do
@@ -444,7 +482,7 @@ INT what;
   } while(k);
   nc = 0;
   for (i=0; i<vc; i++)
-    nc =  triang_getvertexvals(tr,&vv[i*nc],nce[i],what);
-  return((d==2) ? triang_clotoch(vdptr(tr->xxev),vv,nce,nc,xa) :
-                 triang_cubicint(vdptr(tr->xxev),vv,nce,d,nc,xa));
+    nc =  triang_getvertexvals(fp,evs,&vv[i*nc],nce[i],what);
+  return((d==2) ? triang_clotoch(evp(fp),vv,nce,nc,xa) :
+                 triang_cubicint(evp(fp),vv,nce,d,nc,xa));
 }
